@@ -4,10 +4,16 @@ from wechaty_puppet import (    # type: ignore
     FileBox,
 )
 from .accessory import Accessory
-from .wechaty import Wechaty
+from .wechaty import Wechaty, WechatyOptions
 from .user import (
     Contact,
     Favorite,
+    UrlLink,
+    Friendship,
+    Room,
+    MiniProgram,
+    Tag,
+    Image
 )
 from .config import (
     logging,
@@ -20,4 +26,11 @@ __all__ = [
     'Favorite',
     'FileBox',
     'Wechaty',
+    'WechatyOptions',
+    'Image',
+    'UrlLink',
+    'Room',
+    'Friendship',
+    'Tag',
+    'MiniProgram',
 ]
